using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
// ReSharper disable RedundantUsingDirective
using IdeaBlade.Core.ComponentModel.DataAnnotations; //needed in SL
// ReSharper restore RedundantUsingDirective
using IdeaBlade.EntityModel;
using IdeaBlade.Validation;

// ReSharper disable RedundantNameQualifier

namespace AdvancedNorthwindModel {

    [MetadataType(typeof(Employee.EmployeeMetadata))] // Must specify "Employee." no matter what R# says
    public partial class Employee
    {

        [Display(Name = "Full Name", AutoGenerateField = true)]
        public string FullName
        {
            get
            {
                return (EntityAspect.IsNullEntity) ? "[None]" : FirstName + " " + LastName;
            }
        }

        public override string ToString() { return FullName; } // hack

        /// <summary>Gets Orders placed this year (which is always 1998).</summary>
        /// <remarks>
        /// An example of a filtered navigation property: one that actively
        /// filters by some selection criteria.
        /// It returns a "live" collection meaning that the list adjusts
        /// automatically as books that pass the selection criteria appear
        /// and disappear from the entity cache.
        /// See it tested in CacheOnlyQueryTests.
        /// <para>
        /// ToDo: Should be ReadOnly; vulnerable to direct Add and Remove
        /// that could violate list membership rules.
        /// </para>
        /// </remarks>
        [Bindable(false)]
        [Display(Name = "CY Orders", AutoGenerateField = false)]
        public ObservableCollection<Order> CurrentYearOrders
        {
            get
            {
                return _cyOrders ?? CreateCyOrders();
            }
        }

        private ObservableCollection<Order> CreateCyOrders()
        {
            var orderElm = new EntityListManager<Order>(
                EntityAspect.EntityManager,
                o => o.EmployeeID == EmployeeID &&
                     o.OrderDate.HasValue &&
                     o.OrderDate.Value.Year == 1998,
                new EntityProperty[]
            {
                Order.PropertyMetadata.EmployeeID, 
                Order.PropertyMetadata.OrderDate,
            },
                _cyOrders = new ObservableCollection<Order>(),
                true);
            ;
            return _cyOrders;


            // THIS DOESN'T WORK
            //return new RelatedEntityList<Order>(
            //   this.Orders.Where(o => o.OrderDate != null && o.OrderDate.Value.Year == 1998),
            //   PropertyMetadata.Orders.GetEntityReference(this)
            //);
        }

        private ObservableCollection<Order> _cyOrders;

        #region Employee Metadata

        public static class EmployeeMetadata
        {
            [DateTimeRangeVerifier(MinValue = "1/1/2000", MaxValue = "12/31/2010")]
            public static DateTime? HireDate;

            [Display(Name = "Home #", AutoGenerateField = true)]
            [RegexVerifier("Home phone", "USPhone")]
            public static string HomePhone;
        }
        #endregion
    }
}