﻿using System;
using System.Collections.Generic;

// ReSharper disable InconsistentNaming

namespace IdeaBlade.EntityModel {

  public class EntityScalarQueryCoroutineOperation<T> 
    : CoroutineOperationWrapper<EntityScalarQueryCoroutineCompletedEventArgs<T>> {

    public EntityScalarQueryCoroutineOperation(CoroutineOperation triggerOperation,
       Action<EntityScalarQueryCoroutineOperation<T>> callback = null)
      : base(new EntityScalarQueryCoroutineCompletedEventArgs<T>(triggerOperation)) {
      if (null == callback) return;
      Completed += delegate { callback(this); };

    }

    public IList<object> ChangedEntities {
      get {return CompletedEventArgs.ChangedEntities;}
    }

    public IEntityScalarQuery<T> Query {
      get { return CompletedEventArgs.Query; }
    }

    public FetchStrategy ResolvedFetchStrategy {
      get {return CompletedEventArgs.ResolvedFetchStrategy;}
    }

    public T Result { get {return CompletedEventArgs.Result;}
    }

    public bool WasFetched {
      get { return CompletedEventArgs.WasFetched;}
    }
  }

  public class EntityScalarQueryCoroutineCompletedEventArgs<T>
    : CoroutineCompletedEventArgsWrapper<EntityScalarQueryOperation<T>> {

    internal EntityScalarQueryCoroutineCompletedEventArgs(CoroutineOperation triggerOperation)
      : base(triggerOperation) {
      TargetOperationName = String.Format("EntityScalarQueryOperation<{0}>", typeof(T).Name);
    }

    private EntityScalarQueriedEventArgs<T> QueriedEventArgs {
      get { return (EntityScalarQueriedEventArgs<T>) TargetEventArgs; }
    }


    public IList<object> ChangedEntities {
      get {
        EnsureCompletedSuccessfully("ChangedEntities");
        return QueriedEventArgs.ChangedEntities;
      }
    }

    public IEntityScalarQuery<T> Query {
      get {
        EnsureCompletedSuccessfully("Query");
        return QueriedEventArgs.Query;
      }
    }

    public FetchStrategy ResolvedFetchStrategy {
      get {
        EnsureCompletedSuccessfully("ResolvedFetchStrategy");
        return QueriedEventArgs.ResolvedFetchStrategy;
      }
    }

    public T Result {
      get {
        EnsureCompletedSuccessfully("Result");
        return QueriedEventArgs.Result;
      }
    }

    public bool WasFetched {
      get {
        EnsureCompletedSuccessfully("WasFetched");
        return QueriedEventArgs.WasFetched;
      }
    }

  }
}