using System;
using System.Collections.Generic;
using System.Globalization;
using System.Reflection;
using System.Text;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace IdeaBlade.TestFramework {
  internal static class AssertHelper {

    internal static string AppendMessage(this string source, string message, params object[] parameters) {
      string messageString = String.Format(CultureInfo.CurrentCulture, message, parameters);
      return String.Format(CultureInfo.CurrentCulture, "{0} {1}", messageString, source);
    }

#if !SILVERLIGHT

    internal static void HandleFail(string assertionName, string message, params object[] parameters) {
      Type t = typeof(Assert);
      MethodInfo mi = t.GetMethod("HandleFail", BindingFlags.Static | BindingFlags.NonPublic);
      try {
        mi.Invoke(null, new object[] { assertionName, message, parameters });
      } catch (TargetInvocationException ex) {
        throw ex.InnerException;
      }
    }

#else

      // Can't use non-public reflection to get at this method inside 
      // C:\Program Files (x86)\Microsoft SDKs\Silverlight\v4.0\Toolkit\Apr10\Testing\Microsoft.VisualStudio.QualityTools.UnitTesting.Silverlight
      // so get it from Reflector and reimplement here

      internal static void HandleFail(string assertionName, string message, params object[] parameters) {
        string finalMessage = string.Empty;
        if (!string.IsNullOrEmpty(message)) {
          if (parameters == null) {
            finalMessage = ReplaceNulls(message);
          } else {
            finalMessage = string.Format(CultureInfo.CurrentCulture, ReplaceNulls(message), parameters);
          }
        }
        throw new AssertFailedException(AssertionFailed(new object[] { assertionName, finalMessage }));
      }

      internal static string ReplaceNulls(object input) {
        if (input == null) {
          return "(null)".ToString();
        }
        string inputString = input.ToString();
        if (inputString == null) {
          return "(object)".ToString();
        }
        return ReplaceNullChars(inputString);
      }

      public static string ReplaceNullChars(string input) {
        if (string.IsNullOrEmpty(input)) {
          return input;
        }
        List<int> zeroPos = new List<int>();
        for (int i = 0; i < input.Length; i++) {
          if (input[i] == '\0') {
            zeroPos.Add(i);
          }
        }
        if (zeroPos.Count <= 0) {
          return input;
        }
        StringBuilder sb = new StringBuilder(input.Length + zeroPos.Count);
        int start = 0;
        foreach (int index in zeroPos) {
          sb.Append(input.Substring(start, index - start));
          sb.Append(@"\0");
          start = index + 1;
        }
        sb.Append(input.Substring(start));
        return sb.ToString();
      }

      public static string AssertionFailed(params object[] p) {
        return string.Format("{0} failed. {1}", p);
      }

 

#endif


  }
}
