using System;
// ReSharper disable RedundantUsingDirective
using System.ComponentModel; // keep for SL
// ReSharper restore RedundantUsingDirective
using System.Reflection;
using Microsoft.Silverlight.Testing;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using AdvancedNorthwindModel;

// ReSharper disable InconsistentNaming
// ReSharper disable RedundantDefaultFieldInitializer
// ReSharper disable RedundantArgumentName

namespace WorkingWithEntities.BaseClassTests
{
    [TestClass]
    public class When_entities_derived_from_EntityBase
    {
        [TestInitialize]
        public void TestInitialize()
        {
            manager = new AdvancedNorthwindManager(shouldConnect: false);
        }

        [TestMethod]
        [Tag("EntityBase")]
        public void then_can_access_EntityTypeName()
        {
            var cust = new Customer();
            Assert.AreEqual("Customer", GetEntityTypeName(cust));         
        }

        [TestMethod]
        [Tag("EntityBase")]
        public void then_Customer_is_an_Actor()
        {
            var isActor = typeof(Actor).IsAssignableFrom(typeof (Customer));
            Assert.IsTrue(isActor, "Customer should be an Actor");
        }

        [TestMethod]
        [Tag("EntityBase")]
        public void then_Order_is_not_an_Actor()
        {
            var isActor = typeof(Actor).IsAssignableFrom(typeof(Order));
            Assert.IsFalse(isActor, "Order should not be an Actor");
        }

        #region Helpers

        protected string GetEntityTypeName(EntityBase entity)
        {
            var propInfo = typeof(Customer)
                .GetProperty("EntityTypeName", BindingFlags.NonPublic | BindingFlags.Instance);
            return propInfo.GetValue(entity, null) as String;
        }
        protected AdvancedNorthwindManager manager;
        protected Customer testCustomer;

        #endregion

    }

}