﻿using System;
using System.ComponentModel;
using IdeaBlade.Core.ComponentModel;
using IdeaBlade.EntityModel;
using Microsoft.Silverlight.Testing;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using SimpleNorthwindModel;

// ReSharper disable InconsistentNaming
// ReSharper disable RedundantDefaultFieldInitializer
// ReSharper disable RedundantArgumentName

namespace WorkingWithEntities.Validation
{

    [TestClass]
    public class When_validating_customer_in_cache
    {
        [TestInitialize]
        public void TestInitialize()
        {
            manager = new EntityManager(shouldConnect: false);
            testCustomer = new Customer
            {
                CompanyName = "Acme",
                Country = "Freedonia",
            };
            manager.AddEntity(testCustomer);
        }

        [TestMethod]
        [Tag("Validate")]
        public void testCustomer_is_error_free_at_start()
        {
            Assert.IsFalse(testCustIndei.HasErrors, 
                "Should be error free at start");
        }

        /// <summary>Test application of <see cref="NotEmptyGuidVerifier"/></summary>
        /// <remarks>
        /// Get an error because of custom validation rule
        /// <see cref="NotEmptyGuidVerifier"/> added in
        /// <see cref="NorthwindModelVerifierProvider"/>.
        /// </remarks>
        [TestMethod]
        [Tag("Validate")]
        public void has_error_when_set_CustomerID_empty()
        {
            testCustomer.CustomerID = Guid.Empty;
            Assert.IsTrue(testCustIndei.HasErrors, 
                "Should have error when ID is Guid.Empty");
        }

        /// <summary>Test verifier attribute added to Country in partial class</summary>
        [TestMethod]
        [Tag("Validate")]
        public void has_error_when_set_Country_to_empty_string()
        {
            testCustomer.Country = String.Empty;
            Assert.IsTrue(testCustIndei.HasErrors,
                "Should have error when Country is String.Empty");
        }

        protected EntityManager manager;
        protected Customer testCustomer;
        protected IDataErrorInfo testCustIdei { get { return testCustomer; } }
        protected INotifyDataErrorInfo testCustIndei { get { return testCustomer; } }

    }
}