﻿Namespace IdeaBlade.EntityModel

  Public MustInherit Class CoroutineCompletedEventArgsWrapper(Of TOperation As BaseOperation)
	  Inherits CoroutineCompletedEventArgsWrapper

	Protected Sub New(ByVal triggerOperation As CoroutineOperation)
		MyBase.New(triggerOperation)
	  TargetOperationName = GetType(TOperation).Name
	End Sub

	Protected Overrides Sub TriggerOperationCompletedHandler(ByVal sender As Object, ByVal e As CoroutineCompletedEventArgs)
	  If Not e.CompletedSuccessfully Then
		  Return
	  End If
	  Dim targetOp = TryCast(e.Notifications.LastOrDefault(), TOperation)
	  If Nothing Is targetOp Then
		NoTargetEventArgsException = New InvalidOperationException("Successful coroutine's LastCompletedOperation was not a " & TargetOperationName)
		Return
	  End If
	  AddHandler targetOp.Completed, Sub(s, args) TargetEventArgs = args
	End Sub
  End Class

  Public MustInherit Class CoroutineCompletedEventArgsWrapper
	  Inherits EventArgs
	  Implements INotifyCompletedArgs

	Protected Sub New(ByVal triggerOperation As CoroutineOperation)

	  TargetOperationName = "Operation"

	  If Nothing Is triggerOperation Then
		  Throw New ArgumentNullException("triggerOperation")
	  End If
	  Me.TriggerOperation = triggerOperation
	  AddHandler Me.TriggerOperation.Completed, AddressOf TriggerOperationCompletedHandler
	End Sub

	Protected MustOverride Sub TriggerOperationCompletedHandler(ByVal sender As Object, ByVal e As CoroutineCompletedEventArgs)

	Private privateTriggerOperation As CoroutineOperation
	Protected Friend Property TriggerOperation() As CoroutineOperation
		Get
			Return privateTriggerOperation
		End Get
		Private Set(ByVal value As CoroutineOperation)
			privateTriggerOperation = value
		End Set
	End Property
	Protected ReadOnly Property BaseOperation() As BaseOperation
		Get
			Return TriggerOperation
		End Get
	End Property
	Protected Property TargetEventArgs() As EventArgs

	Private privateTargetOperationName As String
	Public Property TargetOperationName() As String
		Get
			Return privateTargetOperationName
		End Get
		Protected Friend Set(ByVal value As String)
			privateTargetOperationName = value
		End Set
	End Property
    Public ReadOnly Property Cancelled() As Boolean Implements INotifyCompletedArgs.Cancelled
      Get
        Return BaseOperation.Cancelled
      End Get
    End Property
	Public ReadOnly Property CompletedSuccessfully() As Boolean
		Get
			Return BaseOperation.CompletedSuccessfully
		End Get
	End Property
	Public ReadOnly Property CompletedSynchronously() As Boolean
		Get
			Return BaseOperation.CompletedSynchronously
		End Get
	End Property
    Public ReadOnly Property [Error]() As Exception Implements INotifyCompletedArgs.Error
      Get
        Return BaseOperation.Error
      End Get
    End Property
	Public ReadOnly Property HasError() As Boolean
		Get
			Return BaseOperation.HasError
		End Get
	End Property
	Public ReadOnly Property IsCompleted() As Boolean
		Get
			Return BaseOperation.IsCompleted
		End Get
	End Property
    Public Property IsErrorHandled() As Boolean Implements INotifyCompletedArgs.IsErrorHandled
      Get
        Return BaseOperation.IsErrorHandled
      End Get
      Set(ByVal value As Boolean)
        BaseOperation.IsErrorHandled = value
      End Set
    End Property
	Public Sub MarkErrorAsHandled()
		BaseOperation.MarkErrorAsHandled()
	End Sub
	Public ReadOnly Property UserState() As Object
		Get
			Return BaseOperation.UserState
		End Get
	End Property

	Protected Property NoTargetEventArgsException() As Exception

	Protected Sub EnsureCompletedSuccessfully(ByVal memberName As String)
	  memberName = If(memberName, "EventArg member")

	  If Not TriggerOperation.IsCompleted Then
		Throw New InvalidOperationException(TargetOperationName & " has not completed; cannot access " & memberName & ".")
	  End If

	  If Not TriggerOperation.CompletedSuccessfully Then
		Throw New InvalidOperationException(TargetOperationName & " did not complete successfully; cannot access " & memberName & ".")
	  End If

	  If Nothing Is TargetEventArgs Then

		If Nothing Is NoTargetEventArgsException Then
		Throw New InvalidOperationException("Successful coroutine for " & TargetOperationName & " inexplicably lacks TargetEventArgs.")
		End If
		Throw NoTargetEventArgsException

	  End If

	End Sub
  End Class
End Namespace