﻿Imports System.Collections.ObjectModel
Imports System.ComponentModel

' ReSharper disable InconsistentNaming

Namespace IdeaBlade.EntityModel

  Public MustInherit Class CoroutineOperationWrapper(Of TEventArgs As CoroutineCompletedEventArgsWrapper)
	  Inherits CoroutineOperationWrapper

	Protected Sub New(ByVal completedEventArgs As TEventArgs)
		MyBase.New(completedEventArgs)
	End Sub

	Protected Shadows ReadOnly Property CompletedEventArgs() As TEventArgs
	  Get
		  Return CType(MyBase.CompletedEventArgs, TEventArgs)
	  End Get
	End Property

	Private Event _completed As EventHandler(Of TEventArgs)
	Public Custom Event Completed As EventHandler(Of TEventArgs)
	  AddHandler(ByVal value As EventHandler(Of TEventArgs))
		If IsCompleted Then
		  value(Me, CompletedEventArgs)
		  Return
		End If
		AddHandler _completed, value
	  End AddHandler
	  RemoveHandler(ByVal value As EventHandler(Of TEventArgs))
		  RemoveHandler _completed, value
	  End RemoveHandler
		RaiseEvent(ByVal sender As Object, ByVal e As TEventArgs)
			RaiseEvent _completed(sender, e)
		End RaiseEvent
	End Event

	Protected Overrides Sub RaiseCompleted(ByVal sender As Object, ByVal args As CoroutineCompletedEventArgs)
	  CallCompletedActions()
      If Nothing Is args Then
        Return
      End If
      RaiseEvent _completed(Me, CompletedEventArgs)
	End Sub
  End Class

  Public MustInherit Class CoroutineOperationWrapper
	  Implements INotifyPropertyChanged, INotifyCompleted

	Protected Sub New(ByVal completedEventArgs As CoroutineCompletedEventArgsWrapper)

	  If Nothing Is completedEventArgs Then
		Throw New ArgumentNullException("completedEventArgs")
	  End If
	  Me.CompletedEventArgs = completedEventArgs
	  TriggerOperation = completedEventArgs.TriggerOperation
	  AddHandler TriggerOperation.Completed, AddressOf RaiseCompleted
	  AddHandler TriggerOperation.PropertyChanged, Sub(s, args) RaisePropertyChanged(args.PropertyName)
	End Sub

	Private privateCompletedEventArgs As CoroutineCompletedEventArgsWrapper
	Protected Property CompletedEventArgs() As CoroutineCompletedEventArgsWrapper
		Get
			Return privateCompletedEventArgs
		End Get
		Private Set(ByVal value As CoroutineCompletedEventArgsWrapper)
			privateCompletedEventArgs = value
		End Set
	End Property
	Private privateTriggerOperation As CoroutineOperation
	Protected Property TriggerOperation() As CoroutineOperation
		Get
			Return privateTriggerOperation
		End Get
		Private Set(ByVal value As CoroutineOperation)
			privateTriggerOperation = value
		End Set
	End Property
	Private ReadOnly Property BaseOperation() As BaseOperation
		Get
			Return TriggerOperation
		End Get
	End Property

	Protected MustOverride Sub RaiseCompleted(ByVal sender As Object, ByVal args As CoroutineCompletedEventArgs)

	Public Property TargetOperationName() As String
	  Get
		  Return CompletedEventArgs.TargetOperationName
	  End Get
	  Set(ByVal value As String)
		  CompletedEventArgs.TargetOperationName = value
	  End Set
	End Property

	Public ReadOnly Property Notifications() As ReadOnlyCollection(Of INotifyCompleted)
	  Get
		  Return TriggerOperation.Notifications
	  End Get
	End Property

	Public ReadOnly Property CanCancel() As Boolean
		Get
			Return BaseOperation.CanCancel
		End Get
	End Property
	Public Sub Cancel()
		BaseOperation.Cancel()
	End Sub
	Public ReadOnly Property Cancelled() As Boolean
		Get
			Return BaseOperation.Cancelled
		End Get
	End Property
	Public ReadOnly Property CompletedSuccessfully() As Boolean
		Get
			Return BaseOperation.CompletedSuccessfully
		End Get
	End Property
	Public ReadOnly Property CompletedSynchronously() As Boolean
		Get
			Return BaseOperation.CompletedSynchronously
		End Get
	End Property
	Public ReadOnly Property [Error]() As Exception
		Get
			Return BaseOperation.Error
		End Get
	End Property
	Public ReadOnly Property HasError() As Boolean
		Get
			Return BaseOperation.HasError
		End Get
	End Property
	Public ReadOnly Property IsCompleted() As Boolean
		Get
			Return BaseOperation.IsCompleted
		End Get
	End Property
	Public Property IsErrorHandled() As Boolean
	  Get
		  Return BaseOperation.IsErrorHandled
	  End Get
	  Set(ByVal value As Boolean)
		  BaseOperation.IsErrorHandled = value
	  End Set
	End Property
	Public Sub MarkErrorAsHandled()
		BaseOperation.MarkErrorAsHandled()
	End Sub
	Public ReadOnly Property UserState() As Object
		Get
			Return BaseOperation.UserState
		End Get
	End Property

	Public Event PropertyChanged As PropertyChangedEventHandler Implements INotifyPropertyChanged.PropertyChanged

	Protected Sub RaisePropertyChanged(ByVal propertyName As String)
	  RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(propertyName))
	End Sub

	Protected Sub CallCompletedActions()
	  Dim actions = _notifyCompletedActions
	  _notifyCompletedActions = Nothing
	  If Nothing Is actions Then
		  Return
	  End If
	  actions(CompletedEventArgs)
	End Sub

	Private _notifyCompletedActions As Action(Of INotifyCompletedArgs)
    Public Overridable Sub WhenCompleted(ByVal completedAction As Action(Of INotifyCompletedArgs)) Implements INotifyCompleted.WhenCompleted
      If Nothing Is completedAction Then
        Return
      End If
      If IsCompleted Then
        completedAction(CompletedEventArgs)
        Return
      End If
      _notifyCompletedActions = CType(System.Delegate.Combine(_notifyCompletedActions, completedAction), Action(Of INotifyCompletedArgs))
    End Sub

  End Class

End Namespace