﻿Imports System.Collections

' ReSharper disable InconsistentNaming

Namespace IdeaBlade.EntityModel

  Public Class EntityQueryCoroutineOperation
	  Inherits CoroutineOperationWrapper(Of EntityQueryCoroutineCompletedEventArgs)

	Public Sub New(ByVal triggerOperation As CoroutineOperation, Optional ByVal callback As Action(Of EntityQueryCoroutineOperation) = Nothing)
		MyBase.New(New EntityQueryCoroutineCompletedEventArgs(triggerOperation))
	  If Nothing Is callback Then
		  Return
	  End If
	  AddHandler Completed, Sub() callback(Me)
	End Sub

	Public ReadOnly Property ChangedEntities() As IList(Of Object)
	  Get
		  Return CompletedEventArgs.ChangedEntities
	  End Get
	End Property

	Public ReadOnly Property EntityQuery() As IEntityQuery
	  Get
		  Return CompletedEventArgs.EntityQuery
	  End Get
	End Property

	Public ReadOnly Property ResolvedFetchStrategy() As FetchStrategy
	  Get
		  Return CompletedEventArgs.ResolvedFetchStrategy
	  End Get
	End Property

	Public ReadOnly Property Results() As IEnumerable
	  Get
		  Return CompletedEventArgs.Results
	  End Get
	End Property

	Public ReadOnly Property WasFetched() As Boolean
	  Get
		  Return CompletedEventArgs.WasFetched
	  End Get
	End Property
  End Class

  Public Class EntityQueryCoroutineCompletedEventArgs
	  Inherits CoroutineCompletedEventArgsWrapper(Of EntityQueryOperation)

	Friend Sub New(ByVal triggerOperation As CoroutineOperation)
		MyBase.New(triggerOperation)
	  TargetOperationName = "EntityQueryOperation"
	End Sub

	Private ReadOnly Property QueriedEventArgs() As EntityQueriedEventArgs
	  Get
		  Return CType(TargetEventArgs, EntityQueriedEventArgs)
	  End Get
	End Property


	Public ReadOnly Property ChangedEntities() As IList(Of Object)
	  Get
		EnsureCompletedSuccessfully("ChangedEntities")
		Return QueriedEventArgs.ChangedEntities
	  End Get
	End Property

	Public ReadOnly Property EntityQuery() As IEntityQuery
	  Get
		EnsureCompletedSuccessfully("Query")
		Return QueriedEventArgs.Query
	  End Get
	End Property

	Public ReadOnly Property ResolvedFetchStrategy() As FetchStrategy
	  Get
		EnsureCompletedSuccessfully("ResolvedFetchStrategy")
		Return QueriedEventArgs.ResolvedFetchStrategy
	  End Get
	End Property

	Public ReadOnly Property Results() As IEnumerable
	  Get
		EnsureCompletedSuccessfully("Results")
		Return QueriedEventArgs.Results
	  End Get
	End Property

	Public ReadOnly Property WasFetched() As Boolean
	  Get
		EnsureCompletedSuccessfully("WasFetched")
		Return QueriedEventArgs.WasFetched
	  End Get
	End Property

  End Class

'------------------------------------------------------------------------------

  Public Class EntityQueryCoroutineOperation(Of T)
	  Inherits CoroutineOperationWrapper(Of EntityQueryCoroutineCompletedEventArgs(Of T))

	Public Sub New(ByVal triggerOperation As CoroutineOperation, Optional ByVal callback As Action(Of EntityQueryCoroutineOperation(Of T)) = Nothing)
		MyBase.New(New EntityQueryCoroutineCompletedEventArgs(Of T)(triggerOperation))
	  If Nothing Is callback Then
		  Return
	  End If
	  AddHandler Completed, Sub() callback(Me)
	End Sub

	Public ReadOnly Property ChangedEntities() As IList(Of Object)
	  Get
		  Return CompletedEventArgs.ChangedEntities
	  End Get
	End Property

	Public ReadOnly Property EntityQuery() As IEntityQuery(Of T)
	  Get
		  Return CompletedEventArgs.EntityQuery
	  End Get
	End Property

	Public ReadOnly Property ResolvedFetchStrategy() As FetchStrategy
	  Get
		  Return CompletedEventArgs.ResolvedFetchStrategy
	  End Get
	End Property

	Public ReadOnly Property Results() As IEnumerable(Of T)
	  Get
		  Return CompletedEventArgs.Results
	  End Get
	End Property

	Public ReadOnly Property WasFetched() As Boolean
	  Get
		  Return CompletedEventArgs.WasFetched
	  End Get
	End Property
  End Class

  Public Class EntityQueryCoroutineCompletedEventArgs(Of T)
	  Inherits CoroutineCompletedEventArgsWrapper(Of EntityQueryOperation(Of T))

	Friend Sub New(ByVal triggerOperation As CoroutineOperation)
		MyBase.New(triggerOperation)
	  TargetOperationName = String.Format("EntityQueryOperation<{0}>", GetType(T).Name)
	End Sub

	Private ReadOnly Property QueriedEventArgs() As EntityQueriedEventArgs(Of T)
	  Get
		  Return CType(TargetEventArgs, EntityQueriedEventArgs(Of T))
	  End Get
	End Property


	Public ReadOnly Property ChangedEntities() As IList(Of Object)
	  Get
		EnsureCompletedSuccessfully("ChangedEntities")
		Return QueriedEventArgs.ChangedEntities
	  End Get
	End Property

	Public ReadOnly Property EntityQuery() As IEntityQuery(Of T)
	  Get
		EnsureCompletedSuccessfully("Query")
		Return QueriedEventArgs.Query
	  End Get
	End Property

	Public ReadOnly Property ResolvedFetchStrategy() As FetchStrategy
	  Get
		EnsureCompletedSuccessfully("ResolvedFetchStrategy")
		Return QueriedEventArgs.ResolvedFetchStrategy
	  End Get
	End Property

	Public ReadOnly Property Results() As IEnumerable(Of T)
	  Get
		EnsureCompletedSuccessfully("Results")
		Return QueriedEventArgs.Results
	  End Get
	End Property

	Public ReadOnly Property WasFetched() As Boolean
	  Get
		EnsureCompletedSuccessfully("WasFetched")
		Return QueriedEventArgs.WasFetched
	  End Get
	End Property

  End Class

End Namespace