' ReSharper disable MemberCanBeMadeStatic.Local

Namespace IdeaBlade.EntityModel

  Public Module EntitySaveOperationExtensions

	Public OperationWrapperFactory As Func(Of EntitySaveOperation, IEntitySaveOperationWrapper) = Function(op) New DefaultEntitySaveOperationWrapper(op)

	Private Function MakeWrapper(ByVal op As EntitySaveOperation) As IEntitySaveOperationWrapper
	  Return OperationWrapperFactory(op)
	End Function

	<System.Runtime.CompilerServices.Extension> _
	Public Function OnCancelled(ByVal op As EntitySaveOperation, ByVal action As Action) As IEntitySaveOperationWrapper
	  Return MakeWrapper(op).OnCancelled(action)
	End Function

	''' <summary>Equivalent to operation's Completed event; terminates the "On..." chain.</summary>
	<System.Runtime.CompilerServices.Extension> _
	Public Sub OnCompleted(ByVal op As EntitySaveOperation, ByVal action As Action(Of EntitySavedEventArgs))
	  MakeWrapper(op).OnCompleted(action)
	End Sub

	<System.Runtime.CompilerServices.Extension> _
	Public Function OnError(ByVal op As EntitySaveOperation, ByVal action As Action(Of Exception)) As IEntitySaveOperationWrapper
	  Return MakeWrapper(op).OnError(action)
	End Function

	<System.Runtime.CompilerServices.Extension> _
	Public Function OnError(ByVal op As EntitySaveOperation, Optional ByVal failureMessageTemplate As String = Nothing) As IEntitySaveOperationWrapper
	  Return MakeWrapper(op).OnError(failureMessageTemplate)
	End Function

	<System.Runtime.CompilerServices.Extension> _
	Public Function OnSuccess(ByVal op As EntitySaveOperation, ByVal action As Action(Of EntitySavedEventArgs)) As IEntitySaveOperationWrapper
	  Return MakeWrapper(op).OnSuccess(action)
	End Function
  End Module

  Public Interface IEntitySaveOperationWrapper

	Function OnSuccess(ByVal action As Action(Of EntitySavedEventArgs)) As IEntitySaveOperationWrapper
	Function OnCancelled(ByVal action As Action) As IEntitySaveOperationWrapper
	''' <summary>Equivalent to operation's Completed event; terminates the "On..." chain.</summary>
	Sub OnCompleted(ByVal action As Action(Of EntitySavedEventArgs))
	Function OnError(ByVal action As Action(Of Exception)) As IEntitySaveOperationWrapper
	Function OnError(Optional ByVal failureMessageTemplate As String = Nothing) As IEntitySaveOperationWrapper

  End Interface

  Public Class DefaultEntitySaveOperationWrapper
	  Implements IEntitySaveOperationWrapper

	Public Sub New(ByVal op As EntitySaveOperation)
	  Operation = op
	  OnCompletedHandler = New OperationOnCompletedHandler(Of EntitySaveOperation, EntitySavedEventArgs)(op)
	  AddHandler op.Completed, AddressOf OnCompletedHandler.CompletedHandler
	End Sub

	Private privateOperation As EntitySaveOperation
	Public Property Operation() As EntitySaveOperation
		Get
			Return privateOperation
		End Get
		Private Set(ByVal value As EntitySaveOperation)
			privateOperation = value
		End Set
	End Property

	Protected Property OnCompletedHandler() As OperationOnCompletedHandler(Of EntitySaveOperation, EntitySavedEventArgs)

	Public Function OnCancelled(ByVal action As Action) As IEntitySaveOperationWrapper Implements IEntitySaveOperationWrapper.OnCancelled
	  OnCompletedHandler.OnCancelled(action)
	  Return Me
	End Function

	Public Sub OnCompleted(ByVal action As Action(Of EntitySavedEventArgs)) Implements IEntitySaveOperationWrapper.OnCompleted
	  OnCompletedHandler.OnCompleted(action)
	End Sub

	Public Function OnError(ByVal action As Action(Of Exception)) As IEntitySaveOperationWrapper Implements IEntitySaveOperationWrapper.OnError
	  OnCompletedHandler.OnError(action)
	  Return Me
	End Function

	Public Function OnError(Optional ByVal failureMessageTemplate As String = Nothing) As IEntitySaveOperationWrapper Implements IEntitySaveOperationWrapper.OnError
	  OnCompletedHandler.OnError(MakeFailureMessageTemplate(failureMessageTemplate))
	  Return Me
	End Function

	Private Function MakeFailureMessageTemplate(ByVal failureMessageTemplate As String) As String
	  If String.IsNullOrWhiteSpace(failureMessageTemplate) Then
		failureMessageTemplate = "Save failed with exception ""{0}""."
	  End If
	  Return failureMessageTemplate
	End Function

	Public Function OnSuccess(ByVal action As Action(Of EntitySavedEventArgs)) As IEntitySaveOperationWrapper Implements IEntitySaveOperationWrapper.OnSuccess
	  OnCompletedHandler.OnSuccess(action)
	  Return Me
	End Function

  End Class

End Namespace