﻿' ReSharper disable InconsistentNaming

Namespace IdeaBlade.EntityModel

  Public Class EntityScalarQueryCoroutineOperation(Of T)
	  Inherits CoroutineOperationWrapper(Of EntityScalarQueryCoroutineCompletedEventArgs(Of T))

	Public Sub New(ByVal triggerOperation As CoroutineOperation, Optional ByVal callback As Action(Of EntityScalarQueryCoroutineOperation(Of T)) = Nothing)
		MyBase.New(New EntityScalarQueryCoroutineCompletedEventArgs(Of T)(triggerOperation))
	  If Nothing Is callback Then
		  Return
	  End If
	  AddHandler Completed, Sub() callback(Me)

	End Sub

	Public ReadOnly Property ChangedEntities() As IList(Of Object)
	  Get
		  Return CompletedEventArgs.ChangedEntities
	  End Get
	End Property

	Public ReadOnly Property Query() As IEntityScalarQuery(Of T)
	  Get
		  Return CompletedEventArgs.Query
	  End Get
	End Property

	Public ReadOnly Property ResolvedFetchStrategy() As FetchStrategy
	  Get
		  Return CompletedEventArgs.ResolvedFetchStrategy
	  End Get
	End Property

	Public ReadOnly Property Result() As T
		Get
			Return CompletedEventArgs.Result
		End Get
	End Property

	Public ReadOnly Property WasFetched() As Boolean
	  Get
		  Return CompletedEventArgs.WasFetched
	  End Get
	End Property
  End Class

  Public Class EntityScalarQueryCoroutineCompletedEventArgs(Of T)
	  Inherits CoroutineCompletedEventArgsWrapper(Of EntityScalarQueryOperation(Of T))

	Friend Sub New(ByVal triggerOperation As CoroutineOperation)
		MyBase.New(triggerOperation)
	  TargetOperationName = String.Format("EntityScalarQueryOperation<{0}>", GetType(T).Name)
	End Sub

	Private ReadOnly Property QueriedEventArgs() As EntityScalarQueriedEventArgs(Of T)
	  Get
		  Return CType(TargetEventArgs, EntityScalarQueriedEventArgs(Of T))
	  End Get
	End Property


	Public ReadOnly Property ChangedEntities() As IList(Of Object)
	  Get
		EnsureCompletedSuccessfully("ChangedEntities")
		Return QueriedEventArgs.ChangedEntities
	  End Get
	End Property

	Public ReadOnly Property Query() As IEntityScalarQuery(Of T)
	  Get
		EnsureCompletedSuccessfully("Query")
		Return QueriedEventArgs.Query
	  End Get
	End Property

	Public ReadOnly Property ResolvedFetchStrategy() As FetchStrategy
	  Get
		EnsureCompletedSuccessfully("ResolvedFetchStrategy")
		Return QueriedEventArgs.ResolvedFetchStrategy
	  End Get
	End Property

	Public ReadOnly Property Result() As T
	  Get
		EnsureCompletedSuccessfully("Result")
		Return QueriedEventArgs.Result
	  End Get
	End Property

	Public ReadOnly Property WasFetched() As Boolean
	  Get
		EnsureCompletedSuccessfully("WasFetched")
		Return QueriedEventArgs.WasFetched
	  End Get
	End Property

  End Class
End Namespace