Namespace IdeaBlade.EntityModel

  Public Module EntityScalarQueryOperationExtensions

	' ToDo: Figure out how to substitute a custom MakeWrapper implementation.

	Private Function MakeWrapper(Of T)(ByVal op As EntityScalarQueryOperation(Of T)) As IEntityScalarQueryOperationWrapper(Of T)
	  Return New DefaultEntityScalarQueryOperationWrapper(Of T)(op)
	End Function

	<System.Runtime.CompilerServices.Extension> _
	Public Function OnCancelled(Of T)(ByVal op As EntityScalarQueryOperation(Of T), ByVal action As Action) As IEntityScalarQueryOperationWrapper(Of T)
	  Return MakeWrapper(op).OnCancelled(action)
	End Function

	''' <summary>Equivalent to operation's Completed event; terminates the "On..." chain.</summary>
	<System.Runtime.CompilerServices.Extension> _
	Public Sub OnCompleted(Of T)(ByVal op As EntityScalarQueryOperation(Of T), ByVal action As Action(Of EntityScalarQueriedEventArgs(Of T)))
	  MakeWrapper(op).OnCompleted(action)
	End Sub

	<System.Runtime.CompilerServices.Extension> _
	Public Function OnError(Of T)(ByVal op As EntityScalarQueryOperation(Of T), ByVal action As Action(Of Exception)) As IEntityScalarQueryOperationWrapper(Of T)
	  Return MakeWrapper(op).OnError(action)
	End Function

	<System.Runtime.CompilerServices.Extension> _
	Public Function OnError(Of T)(ByVal op As EntityScalarQueryOperation(Of T), Optional ByVal failureMessageTemplate As String = Nothing) As IEntityScalarQueryOperationWrapper(Of T)
	  Return MakeWrapper(op).OnError(failureMessageTemplate)
	End Function

	<System.Runtime.CompilerServices.Extension> _
	Public Function OnSuccess(Of T)(ByVal op As EntityScalarQueryOperation(Of T), ByVal action As Action(Of T)) As IEntityScalarQueryOperationWrapper(Of T)
	  Return MakeWrapper(op).OnSuccess(action)
	End Function
  End Module

  Public Interface IEntityScalarQueryOperationWrapper(Of T)

	Function OnSuccess(ByVal action As Action(Of T)) As IEntityScalarQueryOperationWrapper(Of T)
	Function OnCancelled(ByVal action As Action) As IEntityScalarQueryOperationWrapper(Of T)
	''' <summary>Equivalent to operation's Completed event; terminates the "On..." chain.</summary>
	Sub OnCompleted(ByVal action As Action(Of EntityScalarQueriedEventArgs(Of T)))
	Function OnError(ByVal action As Action(Of Exception)) As IEntityScalarQueryOperationWrapper(Of T)
	Function OnError(Optional ByVal failureMessageTemplate As String = Nothing) As IEntityScalarQueryOperationWrapper(Of T)

  End Interface

  Public Class DefaultEntityScalarQueryOperationWrapper(Of T)
	  Implements IEntityScalarQueryOperationWrapper(Of T)

	Public Sub New(ByVal op As EntityScalarQueryOperation(Of T))
	  Operation = op
	  OnCompletedHandler = New OperationOnCompletedHandler(Of EntityScalarQueryOperation(Of T), EntityScalarQueriedEventArgs(Of T))(op)
	  AddHandler op.Completed, AddressOf OnCompletedHandler.CompletedHandler
	End Sub

	Private privateOperation As EntityScalarQueryOperation(Of T)
	Public Property Operation() As EntityScalarQueryOperation(Of T)
		Get
			Return privateOperation
		End Get
		Private Set(ByVal value As EntityScalarQueryOperation(Of T))
			privateOperation = value
		End Set
	End Property

	Protected Property OnCompletedHandler() As OperationOnCompletedHandler(Of EntityScalarQueryOperation(Of T), EntityScalarQueriedEventArgs(Of T))

	Public Function OnCancelled(ByVal action As Action) As IEntityScalarQueryOperationWrapper(Of T) Implements IEntityScalarQueryOperationWrapper(Of T).OnCancelled
	  OnCompletedHandler.OnCancelled(action)
	  Return Me
	End Function

	Public Sub OnCompleted(ByVal action As Action(Of EntityScalarQueriedEventArgs(Of T))) Implements IEntityScalarQueryOperationWrapper(Of T).OnCompleted
	  OnCompletedHandler.OnCompleted(action)
	End Sub

	Public Function OnError(ByVal action As Action(Of Exception)) As IEntityScalarQueryOperationWrapper(Of T) Implements IEntityScalarQueryOperationWrapper(Of T).OnError
	  OnCompletedHandler.OnError(action)
	  Return Me
	End Function

	Public Function OnError(Optional ByVal failureMessageTemplate As String = Nothing) As IEntityScalarQueryOperationWrapper(Of T) Implements IEntityScalarQueryOperationWrapper(Of T).OnError
	  OnCompletedHandler.OnError(MakeFailureMessageTemplate(failureMessageTemplate))
	  Return Me
	End Function

	Private Function MakeFailureMessageTemplate(ByVal failureMessageTemplate As String) As String
	  If String.IsNullOrWhiteSpace(failureMessageTemplate) Then
		Dim queryTypeName = Operation.EntityQuery.ElementType.Name
		failureMessageTemplate = "ScalarQuery<" & queryTypeName & "> failed with exception ""{0}""."
	  End If
	  Return failureMessageTemplate
	End Function

	Public Function OnSuccess(ByVal action As Action(Of T)) As IEntityScalarQueryOperationWrapper(Of T) Implements IEntityScalarQueryOperationWrapper(Of T).OnSuccess
      OnCompletedHandler.OnSuccess(Sub(args) action(args.Result))
	  Return Me
	End Function

  End Class

End Namespace