﻿Namespace IdeaBlade.EntityModel

  ''' <summary>
  ''' Supplements native <see cref="EntityStates"/> with some
  ''' useful combinations. May become part of DevForce someday
  ''' </summary>
  Public NotInheritable Class EntityStates

	''' <summary>
	''' Synthetic EntityState, the bit-wise OR of all states that can appear in entity cache.
	''' </summary>
	Public Const AllCachedStates As EntityState = EntityState.Added Or EntityState.Deleted Or EntityState.Modified Or EntityState.Unchanged

	''' <summary>
	''' Synthetic EntityState, the bit-wise OR of all possible states.
	''' </summary>
	Public Const AllStates As EntityState = AllCachedStates Or EntityState.Detached

	''' <summary>
	''' The entity has been added to the EntityManager.
	''' When saved, the entity will be inserted into the database
	''' </summary>
	Public Const Added As EntityState = EntityState.Added

	''' <summary>
	''' The entity was marked for deletion using the Delete() method.
	''' When saved, the entity will be deleted from the database.
	''' </summary>
	Public Const Deleted As EntityState = EntityState.Deleted

	''' <summary>
	''' The entity does not belong to an EntityManager.
	''' It may have been created but not added to the EntityManager
	''' or it may have been removed from an EntityManager.
	''' </summary>
	Public Const Detached As EntityState = EntityState.Detached

	''' <summary>
	''' The entity has been modified.
	''' When saved, the database will be updated with the changed entity.
	''' </summary>
	Public Const Modified As EntityState = EntityState.Modified

	''' <summary>
	''' The entity has not changed since it was last queried or saved.
	''' </summary>
	Public Const Unchanged As EntityState = EntityState.Unchanged

  End Class
End Namespace