' ReSharper disable MemberCanBeMadeStatic.Local

Namespace IdeaBlade.EntityModel

  Public Module InvokeServerMethodOperationExtensions

	Public OperationWrapperFactory As Func(Of InvokeServerMethodOperation, IInvokeServerMethodOperationWrapper) = Function(op) New DefaultInvokeServerMethodOperationWrapper(op)


	Private Function MakeWrapper(ByVal op As InvokeServerMethodOperation) As IInvokeServerMethodOperationWrapper
	  Return OperationWrapperFactory(op)
	End Function

	<System.Runtime.CompilerServices.Extension> _
	Public Function OnCancelled(ByVal op As InvokeServerMethodOperation, ByVal action As Action) As IInvokeServerMethodOperationWrapper
	  Return MakeWrapper(op).OnCancelled(action)
	End Function

	''' <summary>Equivalent to operation's Completed event; terminates the "On..." chain.</summary>
	<System.Runtime.CompilerServices.Extension> _
	Public Sub OnCompleted(ByVal op As InvokeServerMethodOperation, ByVal action As Action(Of InvokeServerMethodEventArgs))
	  MakeWrapper(op).OnCompleted(action)
	End Sub

	<System.Runtime.CompilerServices.Extension> _
	Public Function OnError(ByVal op As InvokeServerMethodOperation, ByVal action As Action(Of Exception)) As IInvokeServerMethodOperationWrapper
	  Return MakeWrapper(op).OnError(action)
	End Function

	<System.Runtime.CompilerServices.Extension> _
	Public Function OnError(ByVal op As InvokeServerMethodOperation, Optional ByVal failureMessageTemplate As String = Nothing) As IInvokeServerMethodOperationWrapper
	  Return MakeWrapper(op).OnError(failureMessageTemplate)
	End Function

	<System.Runtime.CompilerServices.Extension> _
	Public Function OnSuccess(ByVal op As InvokeServerMethodOperation, ByVal action As Action(Of InvokeServerMethodEventArgs)) As IInvokeServerMethodOperationWrapper
	  Return MakeWrapper(op).OnSuccess(action)
	End Function
  End Module

  Public Interface IInvokeServerMethodOperationWrapper

	Function OnSuccess(ByVal action As Action(Of InvokeServerMethodEventArgs)) As IInvokeServerMethodOperationWrapper
	Function OnCancelled(ByVal action As Action) As IInvokeServerMethodOperationWrapper
	''' <summary>Equivalent to operation's Completed event; terminates the "On..." chain.</summary>
	Sub OnCompleted(ByVal action As Action(Of InvokeServerMethodEventArgs))
	Function OnError(ByVal action As Action(Of Exception)) As IInvokeServerMethodOperationWrapper
	Function OnError(Optional ByVal failureMessageTemplate As String = Nothing) As IInvokeServerMethodOperationWrapper

  End Interface

  Public Class DefaultInvokeServerMethodOperationWrapper
	  Implements IInvokeServerMethodOperationWrapper

	Public Sub New(ByVal op As InvokeServerMethodOperation)
	  Operation = op
	  OnCompletedHandler = New OperationOnCompletedHandler(Of InvokeServerMethodOperation, InvokeServerMethodEventArgs)(op)
	  AddHandler op.Completed, AddressOf OnCompletedHandler.CompletedHandler
	End Sub

	Private privateOperation As InvokeServerMethodOperation
	Public Property Operation() As InvokeServerMethodOperation
		Get
			Return privateOperation
		End Get
		Private Set(ByVal value As InvokeServerMethodOperation)
			privateOperation = value
		End Set
	End Property

	Protected Property OnCompletedHandler() As OperationOnCompletedHandler(Of InvokeServerMethodOperation, InvokeServerMethodEventArgs)

	Public Function OnCancelled(ByVal action As Action) As IInvokeServerMethodOperationWrapper Implements IInvokeServerMethodOperationWrapper.OnCancelled
	  OnCompletedHandler.OnCancelled(action)
	  Return Me
	End Function

	Public Sub OnCompleted(ByVal action As Action(Of InvokeServerMethodEventArgs)) Implements IInvokeServerMethodOperationWrapper.OnCompleted
	  OnCompletedHandler.OnCompleted(action)
	End Sub

	Public Function OnError(ByVal action As Action(Of Exception)) As IInvokeServerMethodOperationWrapper Implements IInvokeServerMethodOperationWrapper.OnError
	  OnCompletedHandler.OnError(action)
	  Return Me
	End Function

	Public Function OnError(Optional ByVal failureMessageTemplate As String = Nothing) As IInvokeServerMethodOperationWrapper Implements IInvokeServerMethodOperationWrapper.OnError
	  OnCompletedHandler.OnError(MakeFailureMessageTemplate(failureMessageTemplate))
	  Return Me
	End Function

	Private Function MakeFailureMessageTemplate(ByVal failureMessageTemplate As String) As String
	  If String.IsNullOrWhiteSpace(failureMessageTemplate) Then
		failureMessageTemplate = "Server Method failed with exception ""{0}""."
	  End If
	  Return failureMessageTemplate
	End Function

	Public Function OnSuccess(ByVal action As Action(Of InvokeServerMethodEventArgs)) As IInvokeServerMethodOperationWrapper Implements IInvokeServerMethodOperationWrapper.OnSuccess
	  OnCompletedHandler.OnSuccess(action)
	  Return Me
	End Function

  End Class

End Namespace