Namespace IdeaBlade.Extensions

	''' <summary>
	''' Replacement for <see cref="Guid.NewGuid"/>
	''' ** ALWAYS USE THIS INSTEAD OF CALLING Guid.NewGuid DIRECTLY
	''' </summary>
	''' <remarks>
	''' Covers the <see cref="Guid.NewGuid"/> static method for getting a
	''' new Guid so that  test and development scenarios can 
	''' control how new Guids are created. See <see cref="NewGuidFn"/>.
	''' </remarks>
	Public NotInheritable Class SystemGuid

		''' <summary>
		''' Gets a new <see cref="Guid"/>
		''' in the manner of <see cref="Guid.NewGuid"/>.
		''' See <see cref="NewGuidFn"/>.
		''' </summary>
		Private Sub New()
		End Sub
		Public Shared Function NewGuid() As Guid
			Return NewGuidFn()
		End Function

		''' <summary>
		''' Function for getting a new <see cref="Guid"/>.
		''' Defaults to <see cref="Guid.NewGuid"/> but
		''' could be replaced in a test or development scenario.
		''' </summary>
    Public Shared NewGuidFn As Func(Of Guid) = Function() Guid.NewGuid

		''' <summary>
		''' Reset <see cref="NewGuidFn"/> to application default
		''' </summary>
		Public Shared Sub ResetNewGuidFn()
      NewGuidFn = Function() Guid.NewGuid
		End Sub

	End Class
End Namespace