Imports System.Globalization
Imports System.Reflection
Imports System.Text
Imports Microsoft.VisualStudio.TestTools.UnitTesting

Namespace IdeaBlade.TestFramework
  Friend Module AssertHelper

	<System.Runtime.CompilerServices.Extension> _
	Friend Function AppendMessage(ByVal source As String, ByVal message As String, ParamArray ByVal parameters() As Object) As String
	  Dim messageString As String = String.Format(CultureInfo.CurrentCulture, message, parameters)
	  Return String.Format(CultureInfo.CurrentCulture, "{0} {1}", messageString, source)
	End Function

#If Not SILVERLIGHT Then

	Friend Sub HandleFail(ByVal assertionName As String, ByVal message As String, ParamArray ByVal parameters() As Object)
	  Dim t As Type = GetType(Assert)
	  Dim mi As MethodInfo = t.GetMethod("HandleFail", BindingFlags.Static Or BindingFlags.NonPublic)
	  Try
		mi.Invoke(Nothing, New Object() { assertionName, message, parameters })
	  Catch ex As TargetInvocationException
		Throw ex.InnerException
	  End Try
	End Sub

#Else

	  ' Can't use non-public reflection to get at this method inside 
	  ' C:\Program Files (x86)\Microsoft SDKs\Silverlight\v4.0\Toolkit\Apr10\Testing\Microsoft.VisualStudio.QualityTools.UnitTesting.Silverlight
	  ' so get it from Reflector and reimplement here

	  Friend Sub HandleFail(ByVal assertionName As String, ByVal message As String, ParamArray ByVal parameters() As Object)
		Dim finalMessage As String = String.Empty
		If Not String.IsNullOrEmpty(message) Then
		  If parameters Is Nothing Then
			finalMessage = ReplaceNulls(message)
		  Else
			finalMessage = String.Format(CultureInfo.CurrentCulture, ReplaceNulls(message), parameters)
		  End If
		End If
		Throw New AssertFailedException(AssertionFailed(New Object() { assertionName, finalMessage }))
	  End Sub

	  Friend Function ReplaceNulls(ByVal input As Object) As String
		If input Is Nothing Then
		  Return "(null)".ToString()
		End If
		Dim inputString As String = input.ToString()
		If inputString Is Nothing Then
		  Return "(object)".ToString()
		End If
		Return ReplaceNullChars(inputString)
	  End Function

	  Public Function ReplaceNullChars(ByVal input As String) As String
		If String.IsNullOrEmpty(input) Then
		  Return input
		End If
		Dim zeroPos As New List(Of Integer)()
		For i As Integer = 0 To input.Length - 1
		  If input.Chars(i) = ControlChars.NullChar Then
			zeroPos.Add(i)
		  End If
		Next i
		If zeroPos.Count <= 0 Then
		  Return input
		End If
		Dim sb As New StringBuilder(input.Length + zeroPos.Count)
		Dim start As Integer = 0
		For Each index As Integer In zeroPos
		  sb.Append(input.Substring(start, index - start))
		  sb.Append("\0")
		  start = index + 1
		Next index
		sb.Append(input.Substring(start))
		Return sb.ToString()
	  End Function

	  Public Function AssertionFailed(ParamArray ByVal p() As Object) As String
		Return String.Format("{0} failed. {1}", p)
	  End Function



#End If


  End Module
End Namespace
