' (c) Copyright IdeaBlade, Inc. and Microsoft Corporation
' Derived from Microsoft Silverlight User Test Framework// (c) Copyright Microsoft Corporation.
' This source is subject to the Microsoft Public License (Ms-PL).
' Please see http://go.microsoft.com/fwlink/?LinkID=131993 for details.
' All other rights reserved.

Imports System.Threading

Namespace IdeaBlade.TestFramework
  ''' <summary>
  ''' A test work item that is complete once the condition is asserted.
  ''' </summary>
  Public Class ConditionalWorkItem
	  Inherits SynchronizedWorkItem

	''' <summary>
	''' The conditional delegate.
	''' </summary>
	Private _delegate As Func(Of Boolean)

	''' <summary>
	''' Construct a new conditional work item.
	''' </summary>
	''' <param name="conditionalMethod">Conditional delegate.</param>
	''' <param name="context">
	''' Optional <see cref="SynchronizationContext"/> in which callback should be executed.
	''' </param>
	Public Sub New(ByVal conditionalMethod As Func(Of Boolean), Optional ByVal context As SynchronizationContext = Nothing)
	  _delegate = conditionalMethod
	  SyncContext = context
	End Sub

	''' <summary>
	''' Invoke the condition, will continue invoking until 
	''' the condition is false.
	''' </summary>
	''' <returns>Completes the invocation once the condition is true.</returns>
	Public Overrides Function Invoke() As Boolean
	  Dim isConditionTrue? As Boolean = False
	  Send(Function(state) isConditionTrue = _delegate(), Nothing)
	  Do While Not isConditionTrue.HasValue
		Thread.Sleep(10) ' wait for SyncContext to evaluate it.
	  Loop

	  If isConditionTrue.Value Then
		WorkItemComplete()
	  End If
	  Return MyBase.Invoke()
	End Function
  End Class
End Namespace