' (c) Copyright Microsoft Corporation.
' This source is subject to the Microsoft Public License (Ms-PL).
' Please see http://go.microsoft.com/fwlink/?LinkID=131993 for details.
' All other rights reserved.


Namespace IdeaBlade.TestFramework
	''' <summary>
	''' Test work item type that does not complete until the sleep time has 
	''' elapsed.  This is NOT a blocking Sleep.
	''' </summary>
	Public Class SleepWorkItem
		Inherits WorkItem
		''' <summary>
		''' The amount of time to delay for.
		''' </summary>
		Private _delay As TimeSpan

		''' <summary>
		''' The DateTime that marks the point in time the task is complete.
		''' </summary>
		Private _expires? As Date

		''' <summary>
		''' Create a new Sleep work item, including the number of 
		''' milliseconds to wait until continuing.
		''' </summary>
		''' <param name="delay">Amount of time to wait/delay.</param>
		Public Sub New(ByVal delay As TimeSpan)
			_delay = delay
			_expires = Nothing
		End Sub

		''' <summary>
		''' On the first time, will calculate the final DateTime.  Otherwise, 
		''' null operation (returns) until that time.
		''' </summary>
		''' <returns>Returns a value indicating whether there is more work to be
		''' done.</returns>
		Public Overrides Function Invoke() As Boolean
			If _expires Is Nothing Then
				_expires = Date.Now.Add(_delay)
			End If
			If Date.Now > _expires Then
				WorkItemComplete()
			End If
			Return MyBase.Invoke()
		End Function
	End Class
End Namespace