'   
'   * All attributes in this file are defined in
'   * and important to the Silverlight Unit Test Framework.
'   * They are mimicked here so that tests can be written to
'   * compile both in Silverlight and Desktop projects.
'   

Namespace Microsoft.Silverlight.Testing

  ''' <summary>
  ''' Indicates that test contains asynchronous statements
  ''' and the test should wait until those statements complete.
  ''' </summary>
  <AttributeUsage(AttributeTargets.Method, AllowMultiple := False)>
  Public Class AsynchronousAttribute
	  Inherits Attribute
  End Class

  ''' <summary>
  ''' Mark a Test Method for filtering by the Test Runner
  ''' </summary>
  <AttributeUsage(AttributeTargets.Method, AllowMultiple := True)>
  Public Class TagAttribute
	  Inherits Attribute
	Public Sub New(ByVal tag As String)
	  Me.Tag = tag
	End Sub

	Private privateTag As String
	Public Property Tag() As String
		Get
			Return privateTag
		End Get
		Private Set(ByVal value As String)
			privateTag = value
		End Set
	End Property
  End Class

  ''' <summary>
  ''' An attribute that contains known bug information that can be logged 
  ''' during a unit test run. 
  ''' 
  ''' When applied to a test method, it will invert the result. This makes it 
  ''' possible to perform automated runs and continuous integration while 
  ''' validating and tracking known issues.
  ''' 
  ''' The attribute should have the Fixed bool set to True, or be removed, 
  ''' once the issue is resolved.
  ''' </summary>
  <AttributeUsage(AttributeTargets.Method, AllowMultiple := True)>
  Public NotInheritable Class BugAttribute
	  Inherits Attribute
	''' <summary>
	''' Gets the bug description.
	''' </summary>
	Private privateDescription As String
	Public Property Description() As String
		Get
			Return privateDescription
		End Get
		Private Set(ByVal value As String)
			privateDescription = value
		End Set
	End Property

	''' <summary>
	''' Gets or sets a value indicating whether the known issue has been 
	''' fixed. If it has, the attribute is ignored and will not affect 
	''' test results.
	''' </summary>
	Public Property Fixed() As Boolean

	''' <summary>
	''' Construct a new BugAttribute with no bug description.
	''' </summary>
	Public Sub New()
	End Sub

	''' <summary>
	''' Construct a new BugAttribute with a bug description.
	''' </summary>
	''' <param name="description">Known issue text.</param>
	Public Sub New(ByVal description As String)
	  Me.Description = description
	End Sub

	''' <summary>
	''' Return the bug information.
	''' </summary>
	''' <returns>Known issue as a string.</returns>
	Public Overrides Function ToString() As String
	  Return If(Fixed, "Fixed: " & Description, Description)
	End Function

  End Class
End Namespace