Imports System.Collections
Imports System.IO
Imports IdeaBlade.Core.Composition
Imports IdeaBlade.EntityModel
Imports Microsoft.VisualStudio.TestTools.UnitTesting

' ReSharper disable InconsistentNaming
' ReSharper disable RedundantArgumentName

Namespace IdeaBlade.TestFramework
  Public Module EntityModelTestExtensions

	''' <summary>
	''' Reset the default FakeBackingStore with the <see cref="EntityCacheState"/>.
	''' </summary>
	Public Sub ResetDefaultFakeBackingStore(ByVal ecs As EntityCacheState)
#If SILVERLIGHT Then
	  Throw New InvalidOperationException("No LocalFakeBackingStore in Silverlight.")
#Else
	  EntityServerFakeBackingStore.Local.GetInstance(CompositionContext.Fake.Name).Restore(ecs)
#End If
	End Sub

	''' <summary>
	''' Prevent Manager from talking to the server.
	''' </summary>
	''' <remarks>
	''' Overdetermined! Going all out to keep manager from talking to a server.
	''' </remarks>
	<System.Runtime.CompilerServices.Extension> _
	Public Sub ForceManagerLocal(ByVal manager As EntityManager)
	  manager.Disconnect()
	  manager.BlockFetching()
	  manager.BlockSaving()
	End Sub

	' Throw if EntityManager attempts to fetch entities from server
	<System.Runtime.CompilerServices.Extension> _
	Public Sub BlockFetching(ByVal manager As EntityManager)
	  RemoveHandler manager.Fetching, AddressOf _blockFetching
	  AddHandler manager.Fetching, AddressOf _blockFetching
	End Sub

	' Remove the block on fetching placed by <see cref="BlockFetching"/>.
	<System.Runtime.CompilerServices.Extension> _
	Public Sub UnblockFetching(ByVal manager As EntityManager)
	  RemoveHandler manager.Fetching, AddressOf _blockFetching
	End Sub

	Private Sub _blockFetching(ByVal sender As Object, ByVal args As EntityFetchingEventArgs)
      Throw New AssertFailedException(args.Query.EntityManager.DataSourceExtension & " should not try to fetch data from the server.")
	End Sub

	' Throw if EntityManager attempts to save 
	<System.Runtime.CompilerServices.Extension> _
	Public Sub BlockSaving(ByVal manager As EntityManager)
	  RemoveHandler manager.Saving, AddressOf _blockSaving
	  AddHandler manager.Saving, AddressOf _blockSaving
	End Sub

	' Remove the block on saving placed by <see cref="BlockSaving"/>.
	<System.Runtime.CompilerServices.Extension> _
	Public Sub UnblockSaving(ByVal manager As EntityManager)
	  RemoveHandler manager.Saving, AddressOf _blockSaving
	End Sub

	Private Sub _blockSaving(ByVal sender As Object, ByVal args As EntitySavingEventArgs)
	  Throw New AssertFailedException("EntityManager should not try to save to the database.")
	End Sub
  End Module
End Namespace