﻿Imports System.ComponentModel
Imports IdeaBlade.EntityModel
Imports Microsoft.Silverlight.Testing
Imports Microsoft.VisualStudio.TestTools.UnitTesting
Imports SimpleNorthwindModel

' ReSharper disable InconsistentNaming
' ReSharper disable RedundantDefaultFieldInitializer
' ReSharper disable RedundantArgumentName

Namespace WorkingWithEntities.UndoTests

	<TestClass>
	Public Class When_undoing_test_customer
		<TestInitialize>
		Public Sub TestInitialize()
			manager = New EntityManager(shouldConnect:= False)
			testCustomer = New Customer With {.CustomerID = Guid.NewGuid()}
			manager.AttachEntity(testCustomer)

			propertyChangedPropertyName = propertyChangedNotRaised
			Dim inpc = (CType(testCustomer, INotifyPropertyChanged))
			AddHandler inpc.PropertyChanged, Sub(s, e) propertyChangedPropertyName = e.PropertyName
		End Sub

		<TestMethod>
		Public Sub then_testCustomer_isUnchanged_at_start()
			Assert.IsTrue(TestCustomerEntityState.IsUnchanged())
		End Sub

		<TestMethod, Tag("Undo")>
		Public Sub then_undo_unmodified_via_entity_raises_PropertyChanged_for_entity()
			testCustomer.EntityAspect.RejectChanges()
			Assert.IsTrue(TestCustomerEntityState.IsUnchanged(), "should be unchanged")
			AssertPropertyChangedForEntireEntity()
		End Sub

		<TestMethod, Tag("Undo")>
		Public Sub then_undo_unmodified_via_manager_does_nothing()
			manager.RejectChanges()
			Assert.IsTrue(TestCustomerEntityState.IsUnchanged(), "should be unchanged")
			Assert.AreEqual(propertyChangedNotRaised, propertyChangedPropertyName, "unexpected propChangedName == " & propertyChangedPropertyName)
		End Sub

		<TestMethod, Tag("Undo")>
		Public Sub can_undo_changes_via_entity()
			testCustomer.CompanyName = "Acme"
			Assert.IsTrue(TestCustomerEntityState.IsModified(), "should be modified")
			testCustomer.EntityAspect.RejectChanges()
			Assert.IsTrue(TestCustomerEntityState.IsUnchanged(), "should be unchanged")
		End Sub

		<TestMethod, Tag("Undo")>
		Public Sub can_undo_changes_via_manager()
			testCustomer.CompanyName = "Acme"
			Assert.IsTrue(TestCustomerEntityState.IsModified(), "should be modified")
			manager.RejectChanges()
			Assert.IsTrue(TestCustomerEntityState.IsUnchanged(), "should be unchanged")
		End Sub

		<TestMethod, Tag("Undo")>
		Public Sub then_undo_changes_via_entity_raises_PropertyChanged_for_entity()
			testCustomer.CompanyName = "Acme" ' make a change
			testCustomer.EntityAspect.RejectChanges()
			AssertPropertyChangedForEntireEntity()
		End Sub

		<TestMethod, Tag("Undo")>
		Public Sub then_undo_changes_via_manager_raises_PropertyChanged_for_entity()
			testCustomer.CompanyName = "Acme" ' make a change
			manager.RejectChanges()
			AssertPropertyChangedForEntireEntity()
		End Sub

		<TestMethod, Tag("Undo")>
		Public Sub then_undo_new_customer_via_entity_raises_PropertyChanged_for_entity()
			testCustomer.EntityAspect.SetAdded() ' pretend is new
			testCustomer.EntityAspect.RejectChanges()
			AssertPropertyChangedForEntireEntity()
		End Sub

		<TestMethod, Tag("Undo")>
		Public Sub then_undo_new_customer_via_manager_raises_PropertyChanged_for_entity()
			testCustomer.EntityAspect.SetAdded() ' pretend is new
			manager.RejectChanges()
			AssertPropertyChangedForEntireEntity()
		End Sub


		<TestMethod, Tag("Undo")>
		Public Sub then_undo_new_customer_detaches_it()
			testCustomer.EntityAspect.SetAdded() ' pretend is new
			testCustomer.EntityAspect.RejectChanges()
			Assert.AreEqual(EntityState.Detached, testCustomer.EntityAspect.EntityState, "Expected detached after RejectChanges to added entity")
		End Sub

		#Region "Private helpers"

		Private ReadOnly Property TestCustomerEntityState() As EntityState
			Get
				Return testCustomer.EntityAspect.EntityState
			End Get
		End Property

		Private Sub AssertPropertyChangedForEntireEntity()
			Assert.AreEqual(expectedPropChangedName, propertyChangedPropertyName, "unexpected propChangedName == " & propertyChangedPropertyName)
		End Sub
		Private testCustomer As Customer
		Private manager As EntityManager

		Private propertyChangedPropertyName As String
		Private Const propertyChangedNotRaised As String = "PropertyChanged not raised"

		' http://msdn.microsoft.com/en-us/library/system.componentmodel.inotifypropertychanged.propertychanged.aspx
		' "The PropertyChanged event can indicate all properties on the object have changed 
		' by using either null or String.Empty as the property name in the PropertyChangedEventArgs."
#If SILVERLIGHT Then
	' The PropertyName for a whole entity change is empty string in DF SL
		Private ReadOnly expectedPropChangedName As String = String.Empty
#Else
		' The PropertyName for a whole entity change is null in DF .NET
		Private Const expectedPropChangedName As String = Nothing
#End If

		#End Region

	End Class
End Namespace